# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Union, Optional
from datetime import datetime
from typing_extensions import Literal, Annotated, TypeAlias

from ..._utils import PropertyInfo
from ..._models import BaseModel
from ..s3_data_source_config import S3DataSourceConfig
from ..local_file_source_config import LocalFileSourceConfig
from ..slack_data_source_config import SlackDataSourceConfig
from ..local_chunks_source_config import LocalChunksSourceConfig
from ..confluence_data_source_config import ConfluenceDataSourceConfig
from ..share_point_data_source_config import SharePointDataSourceConfig
from ..token_chunking_strategy_config import TokenChunkingStrategyConfig
from ..custom_chunking_strategy_config import CustomChunkingStrategyConfig
from ..google_drive_data_source_config import GoogleDriveDataSourceConfig
from ..character_chunking_strategy_config import CharacterChunkingStrategyConfig
from ..azure_blob_storage_data_source_config import AzureBlobStorageDataSourceConfig

__all__ = [
    "KnowledgeBaseUpload",
    "Artifact",
    "ArtifactChunksStatus",
    "ArtifactsStatus",
    "DataSourceConfig",
    "ChunkingStrategyConfig",
]


class ArtifactChunksStatus(BaseModel):
    chunks_completed: int
    """Number of chunks uploaded successfully."""

    chunks_failed: int
    """Number of chunks that failed upload."""

    chunks_pending: int
    """Number of chunks awaiting upload."""


class Artifact(BaseModel):
    artifact_id: str
    """Unique identifier for the artifact."""

    artifact_name: str
    """Friendly name for the artifact."""

    artifact_uri: str
    """Location (e.g. URI) of the artifact in the data source."""

    chunks_status: ArtifactChunksStatus
    """Number of chunks pending, completed, and failed."""

    source: Literal[
        "S3", "Confluence", "SharePoint", "GoogleDrive", "AzureBlobStorage", "Slack", "LocalFile", "LocalChunks"
    ]
    """An enumeration."""

    status: str
    """Status of the artifact."""

    artifact_uri_public: Optional[str] = None
    """Public Location (e.g. URI) of the artifact in the data source."""

    status_reason: Optional[str] = None
    """Reason for the artifact's status."""

    updated_at: Optional[datetime] = None
    """Timestamp at which the artifact was last updated."""


class ArtifactsStatus(BaseModel):
    artifacts_chunking: int
    """Number of artifacts in the chunking state"""

    artifacts_completed: int
    """Number of artifacts uploaded successfully."""

    artifacts_embedding: int
    """Number of artifacts in the embedding state"""

    artifacts_failed: int
    """Number of artifacts that failed while being processed."""

    artifacts_pending: int
    """Previously: Number of artifacts awaiting upload.

    Note that this status will be deprecated soon and should show 0
    """

    artifacts_uploading: int
    """Number of artifacts with upload in progress."""


DataSourceConfig: TypeAlias = Annotated[
    Union[
        S3DataSourceConfig,
        SharePointDataSourceConfig,
        GoogleDriveDataSourceConfig,
        AzureBlobStorageDataSourceConfig,
        LocalChunksSourceConfig,
        LocalFileSourceConfig,
        ConfluenceDataSourceConfig,
        SlackDataSourceConfig,
    ],
    PropertyInfo(discriminator="source"),
]

ChunkingStrategyConfig: TypeAlias = Annotated[
    Union[CharacterChunkingStrategyConfig, TokenChunkingStrategyConfig, CustomChunkingStrategyConfig],
    PropertyInfo(discriminator="strategy"),
]


class KnowledgeBaseUpload(BaseModel):
    artifacts: List[Artifact]
    """List of info for each artifacts associated with this upload.

    This includes artifacts for this data source that are retried.
    """

    artifacts_status: ArtifactsStatus
    """
    Number of artifacts in each of the various states, such as completed and failed
    for this upload. This includes artifacts for this data source that are retried.
    """

    created_at: str
    """The timestamp at which the upload job started."""

    data_source_config: DataSourceConfig
    """Configuration for downloading data from source."""

    status: Literal["Running", "Completed", "Failed", "Canceled"]
    """An enumeration."""

    updated_at: str
    """The timestamp at which the upload job was last updated."""

    upload_id: str
    """Unique ID of the upload job."""

    chunking_strategy_config: Optional[ChunkingStrategyConfig] = None
    """Configuration for chunking the text content of each artifact."""

    created_by_schedule_id: Optional[str] = None
    """Id of the upload schedule that triggered this upload_id.

    Null if triggered manually.
    """

    status_reason: Optional[str] = None
    """Reason for the upload job's status."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Union, Optional
from datetime import datetime
from typing_extensions import Literal, Annotated, TypeAlias

from .._utils import PropertyInfo
from .._models import BaseModel
from .application_configuration import ApplicationConfiguration

__all__ = [
    "ApplicationVariantRetrieveResponse",
    "ApplicationVariantV0Response",
    "ApplicationVariantAgentsServiceResponse",
    "OfflineApplicationVariantResponse",
    "OfflineApplicationVariantResponseConfiguration",
]


class ApplicationVariantV0Response(BaseModel):
    id: str

    account_id: str
    """The ID of the account that owns the given entity."""

    application_spec_id: str

    configuration: ApplicationConfiguration

    created_at: datetime
    """The date and time when the entity was created in ISO format."""

    created_by_user_id: str
    """The user who originally created the entity."""

    name: str

    version: Literal["V0"]

    description: Optional[str] = None
    """Optional description of the application variant"""


class ApplicationVariantAgentsServiceResponse(BaseModel):
    id: str

    account_id: str
    """The ID of the account that owns the given entity."""

    application_spec_id: str

    created_at: datetime
    """The date and time when the entity was created in ISO format."""

    created_by_user_id: str
    """The user who originally created the entity."""

    name: str

    version: Literal["AGENTS_SERVICE"]

    configuration: Optional[object] = None

    description: Optional[str] = None
    """Optional description of the application variant"""


class OfflineApplicationVariantResponseConfiguration(BaseModel):
    metadata: Optional[object] = None
    """User defined metadata about the offline application"""

    output_schema_type: Optional[Literal["completion_only", "context_string", "context_chunks"]] = None
    """An enumeration."""


class OfflineApplicationVariantResponse(BaseModel):
    id: str

    account_id: str
    """The ID of the account that owns the given entity."""

    application_spec_id: str

    configuration: OfflineApplicationVariantResponseConfiguration

    created_at: datetime
    """The date and time when the entity was created in ISO format."""

    created_by_user_id: str
    """The user who originally created the entity."""

    name: str

    version: Literal["OFFLINE"]

    description: Optional[str] = None
    """Optional description of the application variant"""


ApplicationVariantRetrieveResponse: TypeAlias = Annotated[
    Union[ApplicationVariantV0Response, ApplicationVariantAgentsServiceResponse, OfflineApplicationVariantResponse],
    PropertyInfo(discriminator="version"),
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["LocalChunksSourceConfig"]


class LocalChunksSourceConfig(BaseModel):
    artifact_name: str
    """The file name assigned to the artifact, containing a file extension.

    Adding an extension is mandatory, to allow detecting file types for text
    extraction.
    """

    artifact_uri: str
    """
    A unique identifier for an artifact within the knowledge base, such as full path
    in a directory or file system.
    """

    source: Literal["LocalChunks"]

    deduplication_strategy: Optional[Literal["Overwrite", "Fail"]] = None
    """An enumeration."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Required, TypedDict

__all__ = ["EvaluationDatasetApproveBatchParams"]


class EvaluationDatasetApproveBatchParams(TypedDict, total=False):
    autogenerated_draft_test_cases: Required[List[str]]
    """Ids of auto generated draft test cases to be approved."""

    force: bool
    """Force approve a batch of autogenerated test case IDs for the evaluation dataset"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, List

import httpx

from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ..._utils import (
    maybe_transform,
    async_maybe_transform,
)
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...pagination import SyncGenerationJobsPagination, AsyncGenerationJobsPagination
from ..._base_client import AsyncPaginator, make_request_options
from ...types.evaluation_datasets import generation_job_create_params
from ...types.evaluation_datasets.evaluation_dataset_generation_job import EvaluationDatasetGenerationJob
from ...types.evaluation_datasets.evaluation_dataset_generation_job_response import (
    EvaluationDatasetGenerationJobResponse,
)

__all__ = ["GenerationJobsResource", "AsyncGenerationJobsResource"]


class GenerationJobsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> GenerationJobsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python#accessing-raw-response-data-eg-headers
        """
        return GenerationJobsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> GenerationJobsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python#with_streaming_response
        """
        return GenerationJobsResourceWithStreamingResponse(self)

    def create(
        self,
        evaluation_dataset_id: str,
        *,
        advanced_config: Dict[str, List[str]] | NotGiven = NOT_GIVEN,
        custom_instructions: str | NotGiven = NOT_GIVEN,
        group_by_artifact_id: bool | NotGiven = NOT_GIVEN,
        harms_list: List[str] | NotGiven = NOT_GIVEN,
        num_test_cases: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> EvaluationDatasetGenerationJobResponse:
        """
        Create Evaluation Dataset Generation Job

        Args:
          advanced_config: Advanced configuration for the evaluation dataset generation job.

          custom_instructions: Custom instructions for test case generation

          group_by_artifact_id: If this flag is true, for every generated test case, the chunks used to generate
              it will be guaranteed to be from the same document (artifact).

          harms_list: List of harms to be used for the evaluation dataset generation. If not provided,
              generation will use the knowledge base id.

          num_test_cases: Number of test cases to generate for the evaluation dataset

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not evaluation_dataset_id:
            raise ValueError(
                f"Expected a non-empty value for `evaluation_dataset_id` but received {evaluation_dataset_id!r}"
            )
        return self._post(
            f"/v4/evaluation-datasets/{evaluation_dataset_id}/generation-jobs",
            body=maybe_transform(
                {
                    "advanced_config": advanced_config,
                    "custom_instructions": custom_instructions,
                    "group_by_artifact_id": group_by_artifact_id,
                    "harms_list": harms_list,
                    "num_test_cases": num_test_cases,
                },
                generation_job_create_params.GenerationJobCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=EvaluationDatasetGenerationJobResponse,
        )

    def retrieve(
        self,
        generation_job_id: str,
        *,
        evaluation_dataset_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> EvaluationDatasetGenerationJob:
        """
        Get Evaluation Dataset Generation Job

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not evaluation_dataset_id:
            raise ValueError(
                f"Expected a non-empty value for `evaluation_dataset_id` but received {evaluation_dataset_id!r}"
            )
        if not generation_job_id:
            raise ValueError(f"Expected a non-empty value for `generation_job_id` but received {generation_job_id!r}")
        return self._get(
            f"/v4/evaluation-datasets/{evaluation_dataset_id}/generation-jobs/{generation_job_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=EvaluationDatasetGenerationJob,
        )

    def list(
        self,
        evaluation_dataset_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SyncGenerationJobsPagination[EvaluationDatasetGenerationJob]:
        """
        Get Evaluation Dataset Generation Jobs

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not evaluation_dataset_id:
            raise ValueError(
                f"Expected a non-empty value for `evaluation_dataset_id` but received {evaluation_dataset_id!r}"
            )
        return self._get_api_list(
            f"/v4/evaluation-datasets/{evaluation_dataset_id}/generation-jobs",
            page=SyncGenerationJobsPagination[EvaluationDatasetGenerationJob],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=EvaluationDatasetGenerationJob,
        )

    def cancel(
        self,
        generation_job_id: str,
        *,
        evaluation_dataset_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """
        Cancel Evaluation Dataset Generation Job

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not evaluation_dataset_id:
            raise ValueError(
                f"Expected a non-empty value for `evaluation_dataset_id` but received {evaluation_dataset_id!r}"
            )
        if not generation_job_id:
            raise ValueError(f"Expected a non-empty value for `generation_job_id` but received {generation_job_id!r}")
        return self._post(
            f"/v4/evaluation-datasets/{evaluation_dataset_id}/generation-jobs/{generation_job_id}/cancel",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )


class AsyncGenerationJobsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncGenerationJobsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python#accessing-raw-response-data-eg-headers
        """
        return AsyncGenerationJobsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncGenerationJobsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python#with_streaming_response
        """
        return AsyncGenerationJobsResourceWithStreamingResponse(self)

    async def create(
        self,
        evaluation_dataset_id: str,
        *,
        advanced_config: Dict[str, List[str]] | NotGiven = NOT_GIVEN,
        custom_instructions: str | NotGiven = NOT_GIVEN,
        group_by_artifact_id: bool | NotGiven = NOT_GIVEN,
        harms_list: List[str] | NotGiven = NOT_GIVEN,
        num_test_cases: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> EvaluationDatasetGenerationJobResponse:
        """
        Create Evaluation Dataset Generation Job

        Args:
          advanced_config: Advanced configuration for the evaluation dataset generation job.

          custom_instructions: Custom instructions for test case generation

          group_by_artifact_id: If this flag is true, for every generated test case, the chunks used to generate
              it will be guaranteed to be from the same document (artifact).

          harms_list: List of harms to be used for the evaluation dataset generation. If not provided,
              generation will use the knowledge base id.

          num_test_cases: Number of test cases to generate for the evaluation dataset

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not evaluation_dataset_id:
            raise ValueError(
                f"Expected a non-empty value for `evaluation_dataset_id` but received {evaluation_dataset_id!r}"
            )
        return await self._post(
            f"/v4/evaluation-datasets/{evaluation_dataset_id}/generation-jobs",
            body=await async_maybe_transform(
                {
                    "advanced_config": advanced_config,
                    "custom_instructions": custom_instructions,
                    "group_by_artifact_id": group_by_artifact_id,
                    "harms_list": harms_list,
                    "num_test_cases": num_test_cases,
                },
                generation_job_create_params.GenerationJobCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=EvaluationDatasetGenerationJobResponse,
        )

    async def retrieve(
        self,
        generation_job_id: str,
        *,
        evaluation_dataset_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> EvaluationDatasetGenerationJob:
        """
        Get Evaluation Dataset Generation Job

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not evaluation_dataset_id:
            raise ValueError(
                f"Expected a non-empty value for `evaluation_dataset_id` but received {evaluation_dataset_id!r}"
            )
        if not generation_job_id:
            raise ValueError(f"Expected a non-empty value for `generation_job_id` but received {generation_job_id!r}")
        return await self._get(
            f"/v4/evaluation-datasets/{evaluation_dataset_id}/generation-jobs/{generation_job_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=EvaluationDatasetGenerationJob,
        )

    def list(
        self,
        evaluation_dataset_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AsyncPaginator[EvaluationDatasetGenerationJob, AsyncGenerationJobsPagination[EvaluationDatasetGenerationJob]]:
        """
        Get Evaluation Dataset Generation Jobs

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not evaluation_dataset_id:
            raise ValueError(
                f"Expected a non-empty value for `evaluation_dataset_id` but received {evaluation_dataset_id!r}"
            )
        return self._get_api_list(
            f"/v4/evaluation-datasets/{evaluation_dataset_id}/generation-jobs",
            page=AsyncGenerationJobsPagination[EvaluationDatasetGenerationJob],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=EvaluationDatasetGenerationJob,
        )

    async def cancel(
        self,
        generation_job_id: str,
        *,
        evaluation_dataset_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """
        Cancel Evaluation Dataset Generation Job

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not evaluation_dataset_id:
            raise ValueError(
                f"Expected a non-empty value for `evaluation_dataset_id` but received {evaluation_dataset_id!r}"
            )
        if not generation_job_id:
            raise ValueError(f"Expected a non-empty value for `generation_job_id` but received {generation_job_id!r}")
        return await self._post(
            f"/v4/evaluation-datasets/{evaluation_dataset_id}/generation-jobs/{generation_job_id}/cancel",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )


class GenerationJobsResourceWithRawResponse:
    def __init__(self, generation_jobs: GenerationJobsResource) -> None:
        self._generation_jobs = generation_jobs

        self.create = to_raw_response_wrapper(
            generation_jobs.create,
        )
        self.retrieve = to_raw_response_wrapper(
            generation_jobs.retrieve,
        )
        self.list = to_raw_response_wrapper(
            generation_jobs.list,
        )
        self.cancel = to_raw_response_wrapper(
            generation_jobs.cancel,
        )


class AsyncGenerationJobsResourceWithRawResponse:
    def __init__(self, generation_jobs: AsyncGenerationJobsResource) -> None:
        self._generation_jobs = generation_jobs

        self.create = async_to_raw_response_wrapper(
            generation_jobs.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            generation_jobs.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            generation_jobs.list,
        )
        self.cancel = async_to_raw_response_wrapper(
            generation_jobs.cancel,
        )


class GenerationJobsResourceWithStreamingResponse:
    def __init__(self, generation_jobs: GenerationJobsResource) -> None:
        self._generation_jobs = generation_jobs

        self.create = to_streamed_response_wrapper(
            generation_jobs.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            generation_jobs.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            generation_jobs.list,
        )
        self.cancel = to_streamed_response_wrapper(
            generation_jobs.cancel,
        )


class AsyncGenerationJobsResourceWithStreamingResponse:
    def __init__(self, generation_jobs: AsyncGenerationJobsResource) -> None:
        self._generation_jobs = generation_jobs

        self.create = async_to_streamed_response_wrapper(
            generation_jobs.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            generation_jobs.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            generation_jobs.list,
        )
        self.cancel = async_to_streamed_response_wrapper(
            generation_jobs.cancel,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from .models import (
    ModelsResource,
    AsyncModelsResource,
    ModelsResourceWithRawResponse,
    AsyncModelsResourceWithRawResponse,
    ModelsResourceWithStreamingResponse,
    AsyncModelsResourceWithStreamingResponse,
)
from ...types import model_group_list_params, model_group_create_params, model_group_update_params
from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ..._utils import (
    maybe_transform,
    async_maybe_transform,
)
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...pagination import SyncPageResponse, AsyncPageResponse
from ..._base_client import AsyncPaginator, make_request_options
from .usage_statistics import (
    UsageStatisticsResource,
    AsyncUsageStatisticsResource,
    UsageStatisticsResourceWithRawResponse,
    AsyncUsageStatisticsResourceWithRawResponse,
    UsageStatisticsResourceWithStreamingResponse,
    AsyncUsageStatisticsResourceWithStreamingResponse,
)
from ...types.model_group import ModelGroup
from ...types.shared.generic_delete_response import GenericDeleteResponse

__all__ = ["ModelGroupsResource", "AsyncModelGroupsResource"]


class ModelGroupsResource(SyncAPIResource):
    @cached_property
    def models(self) -> ModelsResource:
        return ModelsResource(self._client)

    @cached_property
    def usage_statistics(self) -> UsageStatisticsResource:
        return UsageStatisticsResource(self._client)

    @cached_property
    def with_raw_response(self) -> ModelGroupsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python#accessing-raw-response-data-eg-headers
        """
        return ModelGroupsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ModelGroupsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python#with_streaming_response
        """
        return ModelGroupsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        account_id: str,
        name: str,
        description: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ModelGroup:
        """### Description

        Model groups are grouping models together.

        You can add models with the same
        base_model_id.

        Args:
          account_id: The ID of the account that owns the given entity.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v4/model-groups",
            body=maybe_transform(
                {
                    "account_id": account_id,
                    "name": name,
                    "description": description,
                },
                model_group_create_params.ModelGroupCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ModelGroup,
        )

    def retrieve(
        self,
        model_group_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ModelGroup:
        """
        ### Description

        Gets the details of a model group

        ### Details

        This API can be used to get information about a single model group by ID. To use
        this API, pass in the `id` that was returned from your Create Model Group API
        call as a path parameter.

        Review the response schema to see the fields that will be returned.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not model_group_id:
            raise ValueError(f"Expected a non-empty value for `model_group_id` but received {model_group_id!r}")
        return self._get(
            f"/v4/model-groups/{model_group_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ModelGroup,
        )

    def update(
        self,
        model_group_id: str,
        *,
        description: str | NotGiven = NOT_GIVEN,
        name: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ModelGroup:
        """
        ### Description

        Updates a model group

        ### Details

        This API can be used to update the model group that matches the ID that was
        passed in as a path parameter. To use this API, pass in the `id` that was
        returned from your Create Model Group API call as a path parameter.

        Review the request schema to see the fields that can be updated.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not model_group_id:
            raise ValueError(f"Expected a non-empty value for `model_group_id` but received {model_group_id!r}")
        return self._patch(
            f"/v4/model-groups/{model_group_id}",
            body=maybe_transform(
                {
                    "description": description,
                    "name": name,
                },
                model_group_update_params.ModelGroupUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ModelGroup,
        )

    def list(
        self,
        *,
        account_id: str | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        page: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SyncPageResponse[ModelGroup]:
        """
        ### Description

        Lists all model groups accessible to the user.

        ### Details

        This API can be used to list model groups. If a user has access to multiple
        accounts, all model groups from all accounts the user is associated with will be
        returned.

        Args:
          limit: Maximum number of artifacts to be returned by the given endpoint. Defaults to
              100 and cannot be greater than 10k.

          page: Page number for pagination to be returned by the given endpoint. Starts at page
              1

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/v4/model-groups",
            page=SyncPageResponse[ModelGroup],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "account_id": account_id,
                        "limit": limit,
                        "page": page,
                    },
                    model_group_list_params.ModelGroupListParams,
                ),
            ),
            model=ModelGroup,
        )

    def delete(
        self,
        model_group_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> GenericDeleteResponse:
        """
        ### Description

        Deletes a model group

        ### Details

        This API can be used to delete a model group by ID. To use this API, pass in the
        `id` that was returned from your Create Model Group API call as a path
        parameter.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not model_group_id:
            raise ValueError(f"Expected a non-empty value for `model_group_id` but received {model_group_id!r}")
        return self._delete(
            f"/v4/model-groups/{model_group_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=GenericDeleteResponse,
        )


class AsyncModelGroupsResource(AsyncAPIResource):
    @cached_property
    def models(self) -> AsyncModelsResource:
        return AsyncModelsResource(self._client)

    @cached_property
    def usage_statistics(self) -> AsyncUsageStatisticsResource:
        return AsyncUsageStatisticsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncModelGroupsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python#accessing-raw-response-data-eg-headers
        """
        return AsyncModelGroupsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncModelGroupsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python#with_streaming_response
        """
        return AsyncModelGroupsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        account_id: str,
        name: str,
        description: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ModelGroup:
        """### Description

        Model groups are grouping models together.

        You can add models with the same
        base_model_id.

        Args:
          account_id: The ID of the account that owns the given entity.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v4/model-groups",
            body=await async_maybe_transform(
                {
                    "account_id": account_id,
                    "name": name,
                    "description": description,
                },
                model_group_create_params.ModelGroupCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ModelGroup,
        )

    async def retrieve(
        self,
        model_group_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ModelGroup:
        """
        ### Description

        Gets the details of a model group

        ### Details

        This API can be used to get information about a single model group by ID. To use
        this API, pass in the `id` that was returned from your Create Model Group API
        call as a path parameter.

        Review the response schema to see the fields that will be returned.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not model_group_id:
            raise ValueError(f"Expected a non-empty value for `model_group_id` but received {model_group_id!r}")
        return await self._get(
            f"/v4/model-groups/{model_group_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ModelGroup,
        )

    async def update(
        self,
        model_group_id: str,
        *,
        description: str | NotGiven = NOT_GIVEN,
        name: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ModelGroup:
        """
        ### Description

        Updates a model group

        ### Details

        This API can be used to update the model group that matches the ID that was
        passed in as a path parameter. To use this API, pass in the `id` that was
        returned from your Create Model Group API call as a path parameter.

        Review the request schema to see the fields that can be updated.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not model_group_id:
            raise ValueError(f"Expected a non-empty value for `model_group_id` but received {model_group_id!r}")
        return await self._patch(
            f"/v4/model-groups/{model_group_id}",
            body=await async_maybe_transform(
                {
                    "description": description,
                    "name": name,
                },
                model_group_update_params.ModelGroupUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ModelGroup,
        )

    def list(
        self,
        *,
        account_id: str | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        page: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AsyncPaginator[ModelGroup, AsyncPageResponse[ModelGroup]]:
        """
        ### Description

        Lists all model groups accessible to the user.

        ### Details

        This API can be used to list model groups. If a user has access to multiple
        accounts, all model groups from all accounts the user is associated with will be
        returned.

        Args:
          limit: Maximum number of artifacts to be returned by the given endpoint. Defaults to
              100 and cannot be greater than 10k.

          page: Page number for pagination to be returned by the given endpoint. Starts at page
              1

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/v4/model-groups",
            page=AsyncPageResponse[ModelGroup],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "account_id": account_id,
                        "limit": limit,
                        "page": page,
                    },
                    model_group_list_params.ModelGroupListParams,
                ),
            ),
            model=ModelGroup,
        )

    async def delete(
        self,
        model_group_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> GenericDeleteResponse:
        """
        ### Description

        Deletes a model group

        ### Details

        This API can be used to delete a model group by ID. To use this API, pass in the
        `id` that was returned from your Create Model Group API call as a path
        parameter.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not model_group_id:
            raise ValueError(f"Expected a non-empty value for `model_group_id` but received {model_group_id!r}")
        return await self._delete(
            f"/v4/model-groups/{model_group_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=GenericDeleteResponse,
        )


class ModelGroupsResourceWithRawResponse:
    def __init__(self, model_groups: ModelGroupsResource) -> None:
        self._model_groups = model_groups

        self.create = to_raw_response_wrapper(
            model_groups.create,
        )
        self.retrieve = to_raw_response_wrapper(
            model_groups.retrieve,
        )
        self.update = to_raw_response_wrapper(
            model_groups.update,
        )
        self.list = to_raw_response_wrapper(
            model_groups.list,
        )
        self.delete = to_raw_response_wrapper(
            model_groups.delete,
        )

    @cached_property
    def models(self) -> ModelsResourceWithRawResponse:
        return ModelsResourceWithRawResponse(self._model_groups.models)

    @cached_property
    def usage_statistics(self) -> UsageStatisticsResourceWithRawResponse:
        return UsageStatisticsResourceWithRawResponse(self._model_groups.usage_statistics)


class AsyncModelGroupsResourceWithRawResponse:
    def __init__(self, model_groups: AsyncModelGroupsResource) -> None:
        self._model_groups = model_groups

        self.create = async_to_raw_response_wrapper(
            model_groups.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            model_groups.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            model_groups.update,
        )
        self.list = async_to_raw_response_wrapper(
            model_groups.list,
        )
        self.delete = async_to_raw_response_wrapper(
            model_groups.delete,
        )

    @cached_property
    def models(self) -> AsyncModelsResourceWithRawResponse:
        return AsyncModelsResourceWithRawResponse(self._model_groups.models)

    @cached_property
    def usage_statistics(self) -> AsyncUsageStatisticsResourceWithRawResponse:
        return AsyncUsageStatisticsResourceWithRawResponse(self._model_groups.usage_statistics)


class ModelGroupsResourceWithStreamingResponse:
    def __init__(self, model_groups: ModelGroupsResource) -> None:
        self._model_groups = model_groups

        self.create = to_streamed_response_wrapper(
            model_groups.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            model_groups.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            model_groups.update,
        )
        self.list = to_streamed_response_wrapper(
            model_groups.list,
        )
        self.delete = to_streamed_response_wrapper(
            model_groups.delete,
        )

    @cached_property
    def models(self) -> ModelsResourceWithStreamingResponse:
        return ModelsResourceWithStreamingResponse(self._model_groups.models)

    @cached_property
    def usage_statistics(self) -> UsageStatisticsResourceWithStreamingResponse:
        return UsageStatisticsResourceWithStreamingResponse(self._model_groups.usage_statistics)


class AsyncModelGroupsResourceWithStreamingResponse:
    def __init__(self, model_groups: AsyncModelGroupsResource) -> None:
        self._model_groups = model_groups

        self.create = async_to_streamed_response_wrapper(
            model_groups.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            model_groups.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            model_groups.update,
        )
        self.list = async_to_streamed_response_wrapper(
            model_groups.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            model_groups.delete,
        )

    @cached_property
    def models(self) -> AsyncModelsResourceWithStreamingResponse:
        return AsyncModelsResourceWithStreamingResponse(self._model_groups.models)

    @cached_property
    def usage_statistics(self) -> AsyncUsageStatisticsResourceWithStreamingResponse:
        return AsyncUsageStatisticsResourceWithStreamingResponse(self._model_groups.usage_statistics)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Required, TypedDict

from ..parameter_bindings_param import ParameterBindingsParam

__all__ = ["EmbeddingCreateParams"]


class EmbeddingCreateParams(TypedDict, total=False):
    texts: Required[List[str]]

    model_request_parameters: ParameterBindingsParam



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Any, Iterable, cast, overload
from typing_extensions import Literal

import httpx

from .history import (
    HistoryResource,
    AsyncHistoryResource,
    HistoryResourceWithRawResponse,
    AsyncHistoryResourceWithRawResponse,
    HistoryResourceWithStreamingResponse,
    AsyncHistoryResourceWithStreamingResponse,
)
from ...._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ...._utils import (
    required_args,
    maybe_transform,
    async_maybe_transform,
)
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....pagination import SyncPageResponse, AsyncPageResponse
from ...._base_client import AsyncPaginator, make_request_options
from ....types.evaluation_datasets import (
    test_case_list_params,
    test_case_batch_params,
    test_case_create_params,
    test_case_update_params,
)
from ....types.evaluation_datasets.test_case import TestCase
from ....types.shared.generic_delete_response import GenericDeleteResponse
from ....types.evaluation_datasets.test_case_batch_response import TestCaseBatchResponse

__all__ = ["TestCasesResource", "AsyncTestCasesResource"]


class TestCasesResource(SyncAPIResource):
    __test__ = False

    @cached_property
    def history(self) -> HistoryResource:
        return HistoryResource(self._client)

    @cached_property
    def with_raw_response(self) -> TestCasesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python#accessing-raw-response-data-eg-headers
        """
        return TestCasesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> TestCasesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python#with_streaming_response
        """
        return TestCasesResourceWithStreamingResponse(self)

    def create(
        self,
        evaluation_dataset_id: str,
        *,
        test_case_data: test_case_create_params.TestCaseData,
        account_id: str | NotGiven = NOT_GIVEN,
        chat_history: object | NotGiven = NOT_GIVEN,
        test_case_metadata: object | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> TestCase:
        """
        ### Description

        Create a test case for a selected dataset.

        Args:
          test_case_data: The data for the test case in a format matching the provided schema_type

          account_id: The ID of the account that owns the given entity.

          chat_history: Used for tracking previous chat interactions for multi-chat test cases

          test_case_metadata: Metadata for the test case

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not evaluation_dataset_id:
            raise ValueError(
                f"Expected a non-empty value for `evaluation_dataset_id` but received {evaluation_dataset_id!r}"
            )
        return cast(
            TestCase,
            self._post(
                f"/v4/evaluation-datasets/{evaluation_dataset_id}/test-cases",
                body=maybe_transform(
                    {
                        "test_case_data": test_case_data,
                        "account_id": account_id,
                        "chat_history": chat_history,
                        "test_case_metadata": test_case_metadata,
                    },
                    test_case_create_params.TestCaseCreateParams,
                ),
                options=make_request_options(
                    extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
                ),
                cast_to=cast(Any, TestCase),  # Union types cannot be passed in as arguments in the type system
            ),
        )

    def retrieve(
        self,
        test_case_id: str,
        *,
        evaluation_dataset_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> TestCase:
        """
        ### Description

        Get the test case by its id.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not evaluation_dataset_id:
            raise ValueError(
                f"Expected a non-empty value for `evaluation_dataset_id` but received {evaluation_dataset_id!r}"
            )
        if not test_case_id:
            raise ValueError(f"Expected a non-empty value for `test_case_id` but received {test_case_id!r}")
        return cast(
            TestCase,
            self._get(
                f"/v4/evaluation-datasets/{evaluation_dataset_id}/test-cases/{test_case_id}",
                options=make_request_options(
                    extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
                ),
                cast_to=cast(Any, TestCase),  # Union types cannot be passed in as arguments in the type system
            ),
        )

    @overload
    def update(
        self,
        test_case_id: str,
        *,
        evaluation_dataset_id: str,
        chat_history: object | NotGiven = NOT_GIVEN,
        restore: Literal[False] | NotGiven = NOT_GIVEN,
        test_case_data: test_case_update_params.PartialTestCaseVersionRequestTestCaseData | NotGiven = NOT_GIVEN,
        test_case_metadata: object | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> TestCase:
        """
        ### Description

        Update a test case.

        Args:
          chat_history: Used for tracking previous chat interactions for multi-chat test cases

          restore: Set to true to restore the entity from the database.

          test_case_data: The data for the test case in a format matching the provided schema_type

          test_case_metadata: Metadata for the test case

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    def update(
        self,
        test_case_id: str,
        *,
        evaluation_dataset_id: str,
        restore: Literal[True],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> TestCase:
        """
        ### Description

        Update a test case.

        Args:
          restore: Set to true to restore the entity from the database.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @required_args(["evaluation_dataset_id"], ["evaluation_dataset_id", "restore"])
    def update(
        self,
        test_case_id: str,
        *,
        evaluation_dataset_id: str,
        chat_history: object | NotGiven = NOT_GIVEN,
        restore: Literal[False] | Literal[True] | NotGiven = NOT_GIVEN,
        test_case_data: test_case_update_params.PartialTestCaseVersionRequestTestCaseData | NotGiven = NOT_GIVEN,
        test_case_metadata: object | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> TestCase:
        if not evaluation_dataset_id:
            raise ValueError(
                f"Expected a non-empty value for `evaluation_dataset_id` but received {evaluation_dataset_id!r}"
            )
        if not test_case_id:
            raise ValueError(f"Expected a non-empty value for `test_case_id` but received {test_case_id!r}")
        return cast(
            TestCase,
            self._patch(
                f"/v4/evaluation-datasets/{evaluation_dataset_id}/test-cases/{test_case_id}",
                body=maybe_transform(
                    {
                        "chat_history": chat_history,
                        "restore": restore,
                        "test_case_data": test_case_data,
                        "test_case_metadata": test_case_metadata,
                    },
                    test_case_update_params.TestCaseUpdateParams,
                ),
                options=make_request_options(
                    extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
                ),
                cast_to=cast(Any, TestCase),  # Union types cannot be passed in as arguments in the type system
            ),
        )

    def list(
        self,
        evaluation_dataset_id: str,
        *,
        account_id: str | NotGiven = NOT_GIVEN,
        include_archived: bool | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        page: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SyncPageResponse[TestCase]:
        """
        ### Description

        List all test cases for a selected dataset.

        Args:
          limit: Maximum number of artifacts to be returned by the given endpoint. Defaults to
              100 and cannot be greater than 10k.

          page: Page number for pagination to be returned by the given endpoint. Starts at page
              1

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not evaluation_dataset_id:
            raise ValueError(
                f"Expected a non-empty value for `evaluation_dataset_id` but received {evaluation_dataset_id!r}"
            )
        return self._get_api_list(
            f"/v4/evaluation-datasets/{evaluation_dataset_id}/test-cases",
            page=SyncPageResponse[TestCase],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "account_id": account_id,
                        "include_archived": include_archived,
                        "limit": limit,
                        "page": page,
                    },
                    test_case_list_params.TestCaseListParams,
                ),
            ),
            model=cast(Any, TestCase),  # Union types cannot be passed in as arguments in the type system
        )

    def delete(
        self,
        test_case_id: str,
        *,
        evaluation_dataset_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> GenericDeleteResponse:
        """
        ### Description

        Delete a specified test case -- retaining older version belonging to older
        dataset versions.

        ### Details

        This request shouldn't be used when deleting due to compliance reasons. See Wipe
        action.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not evaluation_dataset_id:
            raise ValueError(
                f"Expected a non-empty value for `evaluation_dataset_id` but received {evaluation_dataset_id!r}"
            )
        if not test_case_id:
            raise ValueError(f"Expected a non-empty value for `test_case_id` but received {test_case_id!r}")
        return self._delete(
            f"/v4/evaluation-datasets/{evaluation_dataset_id}/test-cases/{test_case_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=GenericDeleteResponse,
        )

    def batch(
        self,
        evaluation_dataset_id: str,
        *,
        items: Iterable[test_case_batch_params.Item],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> TestCaseBatchResponse:
        """
        ### Description

        Creates a batch of test cases

        ### Details

        This API can be used to create multiple test cases so users do not have to the
        incur the cost of repeated network calls. To use this API, pass in a list of
        test cases in the request body.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not evaluation_dataset_id:
            raise ValueError(
                f"Expected a non-empty value for `evaluation_dataset_id` but received {evaluation_dataset_id!r}"
            )
        return self._post(
            f"/v4/evaluation-datasets/{evaluation_dataset_id}/test-cases/batch",
            body=maybe_transform(items, Iterable[test_case_batch_params.Item]),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TestCaseBatchResponse,
        )


class AsyncTestCasesResource(AsyncAPIResource):
    @cached_property
    def history(self) -> AsyncHistoryResource:
        return AsyncHistoryResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncTestCasesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python#accessing-raw-response-data-eg-headers
        """
        return AsyncTestCasesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncTestCasesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python#with_streaming_response
        """
        return AsyncTestCasesResourceWithStreamingResponse(self)

    async def create(
        self,
        evaluation_dataset_id: str,
        *,
        test_case_data: test_case_create_params.TestCaseData,
        account_id: str | NotGiven = NOT_GIVEN,
        chat_history: object | NotGiven = NOT_GIVEN,
        test_case_metadata: object | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> TestCase:
        """
        ### Description

        Create a test case for a selected dataset.

        Args:
          test_case_data: The data for the test case in a format matching the provided schema_type

          account_id: The ID of the account that owns the given entity.

          chat_history: Used for tracking previous chat interactions for multi-chat test cases

          test_case_metadata: Metadata for the test case

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not evaluation_dataset_id:
            raise ValueError(
                f"Expected a non-empty value for `evaluation_dataset_id` but received {evaluation_dataset_id!r}"
            )
        return cast(
            TestCase,
            await self._post(
                f"/v4/evaluation-datasets/{evaluation_dataset_id}/test-cases",
                body=await async_maybe_transform(
                    {
                        "test_case_data": test_case_data,
                        "account_id": account_id,
                        "chat_history": chat_history,
                        "test_case_metadata": test_case_metadata,
                    },
                    test_case_create_params.TestCaseCreateParams,
                ),
                options=make_request_options(
                    extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
                ),
                cast_to=cast(Any, TestCase),  # Union types cannot be passed in as arguments in the type system
            ),
        )

    async def retrieve(
        self,
        test_case_id: str,
        *,
        evaluation_dataset_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> TestCase:
        """
        ### Description

        Get the test case by its id.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not evaluation_dataset_id:
            raise ValueError(
                f"Expected a non-empty value for `evaluation_dataset_id` but received {evaluation_dataset_id!r}"
            )
        if not test_case_id:
            raise ValueError(f"Expected a non-empty value for `test_case_id` but received {test_case_id!r}")
        return cast(
            TestCase,
            await self._get(
                f"/v4/evaluation-datasets/{evaluation_dataset_id}/test-cases/{test_case_id}",
                options=make_request_options(
                    extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
                ),
                cast_to=cast(Any, TestCase),  # Union types cannot be passed in as arguments in the type system
            ),
        )

    @overload
    async def update(
        self,
        test_case_id: str,
        *,
        evaluation_dataset_id: str,
        chat_history: object | NotGiven = NOT_GIVEN,
        restore: Literal[False] | NotGiven = NOT_GIVEN,
        test_case_data: test_case_update_params.PartialTestCaseVersionRequestTestCaseData | NotGiven = NOT_GIVEN,
        test_case_metadata: object | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> TestCase:
        """
        ### Description

        Update a test case.

        Args:
          chat_history: Used for tracking previous chat interactions for multi-chat test cases

          restore: Set to true to restore the entity from the database.

          test_case_data: The data for the test case in a format matching the provided schema_type

          test_case_metadata: Metadata for the test case

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    async def update(
        self,
        test_case_id: str,
        *,
        evaluation_dataset_id: str,
        restore: Literal[True],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> TestCase:
        """
        ### Description

        Update a test case.

        Args:
          restore: Set to true to restore the entity from the database.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @required_args(["evaluation_dataset_id"], ["evaluation_dataset_id", "restore"])
    async def update(
        self,
        test_case_id: str,
        *,
        evaluation_dataset_id: str,
        chat_history: object | NotGiven = NOT_GIVEN,
        restore: Literal[False] | Literal[True] | NotGiven = NOT_GIVEN,
        test_case_data: test_case_update_params.PartialTestCaseVersionRequestTestCaseData | NotGiven = NOT_GIVEN,
        test_case_metadata: object | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> TestCase:
        if not evaluation_dataset_id:
            raise ValueError(
                f"Expected a non-empty value for `evaluation_dataset_id` but received {evaluation_dataset_id!r}"
            )
        if not test_case_id:
            raise ValueError(f"Expected a non-empty value for `test_case_id` but received {test_case_id!r}")
        return cast(
            TestCase,
            await self._patch(
                f"/v4/evaluation-datasets/{evaluation_dataset_id}/test-cases/{test_case_id}",
                body=await async_maybe_transform(
                    {
                        "chat_history": chat_history,
                        "restore": restore,
                        "test_case_data": test_case_data,
                        "test_case_metadata": test_case_metadata,
                    },
                    test_case_update_params.TestCaseUpdateParams,
                ),
                options=make_request_options(
                    extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
                ),
                cast_to=cast(Any, TestCase),  # Union types cannot be passed in as arguments in the type system
            ),
        )

    def list(
        self,
        evaluation_dataset_id: str,
        *,
        account_id: str | NotGiven = NOT_GIVEN,
        include_archived: bool | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        page: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AsyncPaginator[TestCase, AsyncPageResponse[TestCase]]:
        """
        ### Description

        List all test cases for a selected dataset.

        Args:
          limit: Maximum number of artifacts to be returned by the given endpoint. Defaults to
              100 and cannot be greater than 10k.

          page: Page number for pagination to be returned by the given endpoint. Starts at page
              1

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not evaluation_dataset_id:
            raise ValueError(
                f"Expected a non-empty value for `evaluation_dataset_id` but received {evaluation_dataset_id!r}"
            )
        return self._get_api_list(
            f"/v4/evaluation-datasets/{evaluation_dataset_id}/test-cases",
            page=AsyncPageResponse[TestCase],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "account_id": account_id,
                        "include_archived": include_archived,
                        "limit": limit,
                        "page": page,
                    },
                    test_case_list_params.TestCaseListParams,
                ),
            ),
            model=cast(Any, TestCase),  # Union types cannot be passed in as arguments in the type system
        )

    async def delete(
        self,
        test_case_id: str,
        *,
        evaluation_dataset_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> GenericDeleteResponse:
        """
        ### Description

        Delete a specified test case -- retaining older version belonging to older
        dataset versions.

        ### Details

        This request shouldn't be used when deleting due to compliance reasons. See Wipe
        action.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not evaluation_dataset_id:
            raise ValueError(
                f"Expected a non-empty value for `evaluation_dataset_id` but received {evaluation_dataset_id!r}"
            )
        if not test_case_id:
            raise ValueError(f"Expected a non-empty value for `test_case_id` but received {test_case_id!r}")
        return await self._delete(
            f"/v4/evaluation-datasets/{evaluation_dataset_id}/test-cases/{test_case_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=GenericDeleteResponse,
        )

    async def batch(
        self,
        evaluation_dataset_id: str,
        *,
        items: Iterable[test_case_batch_params.Item],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> TestCaseBatchResponse:
        """
        ### Description

        Creates a batch of test cases

        ### Details

        This API can be used to create multiple test cases so users do not have to the
        incur the cost of repeated network calls. To use this API, pass in a list of
        test cases in the request body.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not evaluation_dataset_id:
            raise ValueError(
                f"Expected a non-empty value for `evaluation_dataset_id` but received {evaluation_dataset_id!r}"
            )
        return await self._post(
            f"/v4/evaluation-datasets/{evaluation_dataset_id}/test-cases/batch",
            body=await async_maybe_transform(items, Iterable[test_case_batch_params.Item]),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TestCaseBatchResponse,
        )


class TestCasesResourceWithRawResponse:
    __test__ = False

    def __init__(self, test_cases: TestCasesResource) -> None:
        self._test_cases = test_cases

        self.create = to_raw_response_wrapper(
            test_cases.create,
        )
        self.retrieve = to_raw_response_wrapper(
            test_cases.retrieve,
        )
        self.update = to_raw_response_wrapper(
            test_cases.update,
        )
        self.list = to_raw_response_wrapper(
            test_cases.list,
        )
        self.delete = to_raw_response_wrapper(
            test_cases.delete,
        )
        self.batch = to_raw_response_wrapper(
            test_cases.batch,
        )

    @cached_property
    def history(self) -> HistoryResourceWithRawResponse:
        return HistoryResourceWithRawResponse(self._test_cases.history)


class AsyncTestCasesResourceWithRawResponse:
    def __init__(self, test_cases: AsyncTestCasesResource) -> None:
        self._test_cases = test_cases

        self.create = async_to_raw_response_wrapper(
            test_cases.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            test_cases.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            test_cases.update,
        )
        self.list = async_to_raw_response_wrapper(
            test_cases.list,
        )
        self.delete = async_to_raw_response_wrapper(
            test_cases.delete,
        )
        self.batch = async_to_raw_response_wrapper(
            test_cases.batch,
        )

    @cached_property
    def history(self) -> AsyncHistoryResourceWithRawResponse:
        return AsyncHistoryResourceWithRawResponse(self._test_cases.history)


class TestCasesResourceWithStreamingResponse:
    __test__ = False

    def __init__(self, test_cases: TestCasesResource) -> None:
        self._test_cases = test_cases

        self.create = to_streamed_response_wrapper(
            test_cases.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            test_cases.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            test_cases.update,
        )
        self.list = to_streamed_response_wrapper(
            test_cases.list,
        )
        self.delete = to_streamed_response_wrapper(
            test_cases.delete,
        )
        self.batch = to_streamed_response_wrapper(
            test_cases.batch,
        )

    @cached_property
    def history(self) -> HistoryResourceWithStreamingResponse:
        return HistoryResourceWithStreamingResponse(self._test_cases.history)


class AsyncTestCasesResourceWithStreamingResponse:
    def __init__(self, test_cases: AsyncTestCasesResource) -> None:
        self._test_cases = test_cases

        self.create = async_to_streamed_response_wrapper(
            test_cases.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            test_cases.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            test_cases.update,
        )
        self.list = async_to_streamed_response_wrapper(
            test_cases.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            test_cases.delete,
        )
        self.batch = async_to_streamed_response_wrapper(
            test_cases.batch,
        )

    @cached_property
    def history(self) -> AsyncHistoryResourceWithStreamingResponse:
        return AsyncHistoryResourceWithStreamingResponse(self._test_cases.history)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Union
from typing_extensions import Literal

import httpx

from ..types import (
    application_variant_report_list_params,
    application_variant_report_create_params,
    application_variant_report_retrieve_params,
)
from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._utils import (
    maybe_transform,
    async_maybe_transform,
)
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..pagination import SyncPageResponse, AsyncPageResponse
from .._base_client import AsyncPaginator, make_request_options
from ..types.application_variant_with_scores import ApplicationVariantWithScores
from ..types.application_variant_with_scores_and_views import ApplicationVariantWithScoresAndViews

__all__ = ["ApplicationVariantReportsResource", "AsyncApplicationVariantReportsResource"]


class ApplicationVariantReportsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ApplicationVariantReportsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python#accessing-raw-response-data-eg-headers
        """
        return ApplicationVariantReportsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ApplicationVariantReportsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python#with_streaming_response
        """
        return ApplicationVariantReportsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        application_variant_id: str,
        evaluation_dataset_ids: List[str],
        account_id: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ApplicationVariantWithScores:
        """
        ### Description

        Creates a application variant report

        ### Details

        This API can be used to create a application variant report. To use this API,
        review the request schema and pass in all fields that are required to create a
        application variant report.

        Args:
          account_id: The ID of the account that owns the given entity.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v4/application-variant-reports",
            body=maybe_transform(
                {
                    "application_variant_id": application_variant_id,
                    "evaluation_dataset_ids": evaluation_dataset_ids,
                    "account_id": account_id,
                },
                application_variant_report_create_params.ApplicationVariantReportCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ApplicationVariantWithScores,
        )

    def retrieve(
        self,
        application_variant_report_id: str,
        *,
        view: List[Literal["AsyncJobs"]] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ApplicationVariantWithScoresAndViews:
        """
        ### Description

        Gets the details of a application variant report

        ### Details

        This API can be used to get information about a single application variant
        report by ID. To use this API, pass in the `id` that was returned from your
        Create Application Variant Report API call as a path parameter.

        Review the response schema to see the fields that will be returned.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not application_variant_report_id:
            raise ValueError(
                f"Expected a non-empty value for `application_variant_report_id` but received {application_variant_report_id!r}"
            )
        return self._get(
            f"/v4/application-variant-reports/{application_variant_report_id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {"view": view}, application_variant_report_retrieve_params.ApplicationVariantReportRetrieveParams
                ),
            ),
            cast_to=ApplicationVariantWithScoresAndViews,
        )

    def list(
        self,
        *,
        account_id: str | NotGiven = NOT_GIVEN,
        application_spec_id: Union[int, str] | NotGiven = NOT_GIVEN,
        application_variant_id: Union[int, str] | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        page: int | NotGiven = NOT_GIVEN,
        view: List[Literal["AsyncJobs"]] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SyncPageResponse[ApplicationVariantWithScoresAndViews]:
        """
        ### Description

        Lists all application variant reports accessible to the user.

        ### Details

        This API can be used to list application variant reports. If a user has access
        to multiple accounts, all application variant reports from all accounts the user
        is associated with will be returned.

        Args:
          limit: Maximum number of artifacts to be returned by the given endpoint. Defaults to
              100 and cannot be greater than 10k.

          page: Page number for pagination to be returned by the given endpoint. Starts at page
              1

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/v4/application-variant-reports",
            page=SyncPageResponse[ApplicationVariantWithScoresAndViews],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "account_id": account_id,
                        "application_spec_id": application_spec_id,
                        "application_variant_id": application_variant_id,
                        "limit": limit,
                        "page": page,
                        "view": view,
                    },
                    application_variant_report_list_params.ApplicationVariantReportListParams,
                ),
            ),
            model=ApplicationVariantWithScoresAndViews,
        )


class AsyncApplicationVariantReportsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncApplicationVariantReportsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python#accessing-raw-response-data-eg-headers
        """
        return AsyncApplicationVariantReportsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncApplicationVariantReportsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python#with_streaming_response
        """
        return AsyncApplicationVariantReportsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        application_variant_id: str,
        evaluation_dataset_ids: List[str],
        account_id: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ApplicationVariantWithScores:
        """
        ### Description

        Creates a application variant report

        ### Details

        This API can be used to create a application variant report. To use this API,
        review the request schema and pass in all fields that are required to create a
        application variant report.

        Args:
          account_id: The ID of the account that owns the given entity.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v4/application-variant-reports",
            body=await async_maybe_transform(
                {
                    "application_variant_id": application_variant_id,
                    "evaluation_dataset_ids": evaluation_dataset_ids,
                    "account_id": account_id,
                },
                application_variant_report_create_params.ApplicationVariantReportCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ApplicationVariantWithScores,
        )

    async def retrieve(
        self,
        application_variant_report_id: str,
        *,
        view: List[Literal["AsyncJobs"]] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ApplicationVariantWithScoresAndViews:
        """
        ### Description

        Gets the details of a application variant report

        ### Details

        This API can be used to get information about a single application variant
        report by ID. To use this API, pass in the `id` that was returned from your
        Create Application Variant Report API call as a path parameter.

        Review the response schema to see the fields that will be returned.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not application_variant_report_id:
            raise ValueError(
                f"Expected a non-empty value for `application_variant_report_id` but received {application_variant_report_id!r}"
            )
        return await self._get(
            f"/v4/application-variant-reports/{application_variant_report_id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"view": view}, application_variant_report_retrieve_params.ApplicationVariantReportRetrieveParams
                ),
            ),
            cast_to=ApplicationVariantWithScoresAndViews,
        )

    def list(
        self,
        *,
        account_id: str | NotGiven = NOT_GIVEN,
        application_spec_id: Union[int, str] | NotGiven = NOT_GIVEN,
        application_variant_id: Union[int, str] | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        page: int | NotGiven = NOT_GIVEN,
        view: List[Literal["AsyncJobs"]] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AsyncPaginator[ApplicationVariantWithScoresAndViews, AsyncPageResponse[ApplicationVariantWithScoresAndViews]]:
        """
        ### Description

        Lists all application variant reports accessible to the user.

        ### Details

        This API can be used to list application variant reports. If a user has access
        to multiple accounts, all application variant reports from all accounts the user
        is associated with will be returned.

        Args:
          limit: Maximum number of artifacts to be returned by the given endpoint. Defaults to
              100 and cannot be greater than 10k.

          page: Page number for pagination to be returned by the given endpoint. Starts at page
              1

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/v4/application-variant-reports",
            page=AsyncPageResponse[ApplicationVariantWithScoresAndViews],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "account_id": account_id,
                        "application_spec_id": application_spec_id,
                        "application_variant_id": application_variant_id,
                        "limit": limit,
                        "page": page,
                        "view": view,
                    },
                    application_variant_report_list_params.ApplicationVariantReportListParams,
                ),
            ),
            model=ApplicationVariantWithScoresAndViews,
        )


class ApplicationVariantReportsResourceWithRawResponse:
    def __init__(self, application_variant_reports: ApplicationVariantReportsResource) -> None:
        self._application_variant_reports = application_variant_reports

        self.create = to_raw_response_wrapper(
            application_variant_reports.create,
        )
        self.retrieve = to_raw_response_wrapper(
            application_variant_reports.retrieve,
        )
        self.list = to_raw_response_wrapper(
            application_variant_reports.list,
        )


class AsyncApplicationVariantReportsResourceWithRawResponse:
    def __init__(self, application_variant_reports: AsyncApplicationVariantReportsResource) -> None:
        self._application_variant_reports = application_variant_reports

        self.create = async_to_raw_response_wrapper(
            application_variant_reports.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            application_variant_reports.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            application_variant_reports.list,
        )


class ApplicationVariantReportsResourceWithStreamingResponse:
    def __init__(self, application_variant_reports: ApplicationVariantReportsResource) -> None:
        self._application_variant_reports = application_variant_reports

        self.create = to_streamed_response_wrapper(
            application_variant_reports.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            application_variant_reports.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            application_variant_reports.list,
        )


class AsyncApplicationVariantReportsResourceWithStreamingResponse:
    def __init__(self, application_variant_reports: AsyncApplicationVariantReportsResource) -> None:
        self._application_variant_reports = application_variant_reports

        self.create = async_to_streamed_response_wrapper(
            application_variant_reports.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            application_variant_reports.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            application_variant_reports.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Union, Optional
from datetime import datetime
from typing_extensions import Literal, Annotated, TypeAlias

from ..._utils import PropertyInfo
from ..._models import BaseModel
from .flexible_test_case_schema import FlexibleTestCaseSchema
from .artifact_schema_generation import ArtifactSchemaGeneration
from .generation_test_case_schema import GenerationTestCaseSchema

__all__ = [
    "TestCase",
    "GenerationTestCaseVersionResponse",
    "GenerationTestCaseVersionResponseTestCaseData",
    "FlexibleTestCaseVersionResponse",
]

GenerationTestCaseVersionResponseTestCaseData: TypeAlias = Union[ArtifactSchemaGeneration, GenerationTestCaseSchema]


class GenerationTestCaseVersionResponse(BaseModel):
    id: str
    """The unique identifier of the entity."""

    account_id: str
    """The ID of the account that owns the given entity."""

    autogenerated: bool
    """Boolean to track whether or not the test case is autogenerated"""

    created_at: datetime
    """The date and time when the entity was created in ISO format."""

    created_by_user_id: str
    """The user who originally created the entity."""

    evaluation_dataset_id: str
    """The ID of the associated evaluation dataset."""

    test_case_data: GenerationTestCaseVersionResponseTestCaseData

    archived_at: Optional[datetime] = None
    """The date and time when the entity was archived in ISO format."""

    chat_history: Optional[object] = None
    """Used for tracking previous chat interactions for multi-chat test cases"""

    schema_type: Optional[Literal["GENERATION"]] = None

    test_case_metadata: Optional[object] = None
    """Metadata for the test case"""


class FlexibleTestCaseVersionResponse(BaseModel):
    id: str
    """The unique identifier of the entity."""

    account_id: str
    """The ID of the account that owns the given entity."""

    autogenerated: bool
    """Boolean to track whether or not the test case is autogenerated"""

    created_at: datetime
    """The date and time when the entity was created in ISO format."""

    created_by_user_id: str
    """The user who originally created the entity."""

    evaluation_dataset_id: str
    """The ID of the associated evaluation dataset."""

    test_case_data: FlexibleTestCaseSchema

    archived_at: Optional[datetime] = None
    """The date and time when the entity was archived in ISO format."""

    chat_history: Optional[object] = None
    """Used for tracking previous chat interactions for multi-chat test cases"""

    schema_type: Optional[Literal["FLEXIBLE"]] = None

    test_case_metadata: Optional[object] = None
    """Metadata for the test case"""


TestCase: TypeAlias = Annotated[
    Union[GenerationTestCaseVersionResponse, FlexibleTestCaseVersionResponse], PropertyInfo(discriminator="schema_type")
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Required, TypedDict

from .._types import FileTypes

__all__ = ["ApplicationUploadFilesParams"]


class ApplicationUploadFilesParams(TypedDict, total=False):
    files: Required[List[FileTypes]]
    """Upload files to be used in an application."""

    account_id: str
    """Account which the file will be tied to.

    Use this account id query param if you are using the API or the SDK.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ..._utils import maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...pagination import SyncTopLevelArray, AsyncTopLevelArray
from ..._base_client import AsyncPaginator, make_request_options
from ...types.knowledge_bases import async_job_list_params
from ...types.knowledge_bases.async_job_list_response import AsyncJobListResponse

__all__ = ["AsyncJobsResource", "AsyncAsyncJobsResource"]


class AsyncJobsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncJobsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python#accessing-raw-response-data-eg-headers
        """
        return AsyncJobsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncJobsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python#with_streaming_response
        """
        return AsyncJobsResourceWithStreamingResponse(self)

    def list(
        self,
        knowledge_base_id: str,
        *,
        limit: int | NotGiven = NOT_GIVEN,
        page: int | NotGiven = NOT_GIVEN,
        status: Literal["Active", "All", "Pending", "Running", "Completed", "Failed", "Canceled"]
        | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SyncTopLevelArray[AsyncJobListResponse]:
        """
        List Upload Jobs

        Args:
          limit: Maximum number of artifacts to be returned by the given endpoint. Defaults to
              100 and cannot be greater than 10k.

          page: Page number for pagination to be returned by the given endpoint. Starts at page
              1

          status: Optional search by status type

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not knowledge_base_id:
            raise ValueError(f"Expected a non-empty value for `knowledge_base_id` but received {knowledge_base_id!r}")
        return self._get_api_list(
            f"/v4/knowledge-bases/{knowledge_base_id}/async-jobs",
            page=SyncTopLevelArray[AsyncJobListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "limit": limit,
                        "page": page,
                        "status": status,
                    },
                    async_job_list_params.AsyncJobListParams,
                ),
            ),
            model=AsyncJobListResponse,
        )


class AsyncAsyncJobsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncAsyncJobsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python#accessing-raw-response-data-eg-headers
        """
        return AsyncAsyncJobsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncAsyncJobsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python#with_streaming_response
        """
        return AsyncAsyncJobsResourceWithStreamingResponse(self)

    def list(
        self,
        knowledge_base_id: str,
        *,
        limit: int | NotGiven = NOT_GIVEN,
        page: int | NotGiven = NOT_GIVEN,
        status: Literal["Active", "All", "Pending", "Running", "Completed", "Failed", "Canceled"]
        | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AsyncPaginator[AsyncJobListResponse, AsyncTopLevelArray[AsyncJobListResponse]]:
        """
        List Upload Jobs

        Args:
          limit: Maximum number of artifacts to be returned by the given endpoint. Defaults to
              100 and cannot be greater than 10k.

          page: Page number for pagination to be returned by the given endpoint. Starts at page
              1

          status: Optional search by status type

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not knowledge_base_id:
            raise ValueError(f"Expected a non-empty value for `knowledge_base_id` but received {knowledge_base_id!r}")
        return self._get_api_list(
            f"/v4/knowledge-bases/{knowledge_base_id}/async-jobs",
            page=AsyncTopLevelArray[AsyncJobListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "limit": limit,
                        "page": page,
                        "status": status,
                    },
                    async_job_list_params.AsyncJobListParams,
                ),
            ),
            model=AsyncJobListResponse,
        )


class AsyncJobsResourceWithRawResponse:
    def __init__(self, async_jobs: AsyncJobsResource) -> None:
        self._async_jobs = async_jobs

        self.list = to_raw_response_wrapper(
            async_jobs.list,
        )


class AsyncAsyncJobsResourceWithRawResponse:
    def __init__(self, async_jobs: AsyncAsyncJobsResource) -> None:
        self._async_jobs = async_jobs

        self.list = async_to_raw_response_wrapper(
            async_jobs.list,
        )


class AsyncJobsResourceWithStreamingResponse:
    def __init__(self, async_jobs: AsyncJobsResource) -> None:
        self._async_jobs = async_jobs

        self.list = to_streamed_response_wrapper(
            async_jobs.list,
        )


class AsyncAsyncJobsResourceWithStreamingResponse:
    def __init__(self, async_jobs: AsyncAsyncJobsResource) -> None:
        self._async_jobs = async_jobs

        self.list = async_to_streamed_response_wrapper(
            async_jobs.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from scale_gp import SGPClient, AsyncSGPClient
from tests.utils import assert_matches_type
from scale_gp.types import (
    ApplicationUploadFilesResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestApplications:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="additional_properties is an object")
    @parametrize
    def test_method_process(self, client: SGPClient) -> None:
        application = client.applications.process(
            edges=[
                {
                    "from_field": "from_field",
                    "from_node": "from_node",
                    "to_field": "to_field",
                    "to_node": "to_node",
                },
                {
                    "from_field": "from_field",
                    "from_node": "from_node",
                    "to_field": "to_field",
                    "to_node": "to_node",
                },
                {
                    "from_field": "from_field",
                    "from_node": "from_node",
                    "to_field": "to_field",
                    "to_node": "to_node",
                },
            ],
            inputs={"foo": "bar"},
            nodes=[
                {
                    "id": "id",
                    "application_node_schema_id": "text_input_schema",
                },
                {
                    "id": "id",
                    "application_node_schema_id": "text_input_schema",
                },
                {
                    "id": "id",
                    "application_node_schema_id": "text_input_schema",
                },
            ],
            version="V0",
        )
        assert_matches_type(object, application, path=["response"])

    @pytest.mark.skip(reason="additional_properties is an object")
    @parametrize
    def test_method_process_with_all_params(self, client: SGPClient) -> None:
        application = client.applications.process(
            edges=[
                {
                    "from_field": "from_field",
                    "from_node": "from_node",
                    "to_field": "to_field",
                    "to_node": "to_node",
                },
                {
                    "from_field": "from_field",
                    "from_node": "from_node",
                    "to_field": "to_field",
                    "to_node": "to_node",
                },
                {
                    "from_field": "from_field",
                    "from_node": "from_node",
                    "to_field": "to_field",
                    "to_node": "to_node",
                },
            ],
            inputs={"foo": "bar"},
            nodes=[
                {
                    "id": "id",
                    "application_node_schema_id": "text_input_schema",
                    "configuration": {"foo": {"value": {}}},
                },
                {
                    "id": "id",
                    "application_node_schema_id": "text_input_schema",
                    "configuration": {"foo": {"value": {}}},
                },
                {
                    "id": "id",
                    "application_node_schema_id": "text_input_schema",
                    "configuration": {"foo": {"value": {}}},
                },
            ],
            version="V0",
            history=[
                {
                    "request": "request",
                    "response": "response",
                    "session_data": {},
                },
                {
                    "request": "request",
                    "response": "response",
                    "session_data": {},
                },
                {
                    "request": "request",
                    "response": "response",
                    "session_data": {},
                },
            ],
            overrides={
                "foo": {
                    "artifact_ids_filter": ["string", "string", "string"],
                    "artifact_name_regex": ["string", "string", "string"],
                    "type": "knowledge_base_schema",
                }
            },
            stream=True,
        )
        assert_matches_type(object, application, path=["response"])

    @pytest.mark.skip(reason="additional_properties is an object")
    @parametrize
    def test_raw_response_process(self, client: SGPClient) -> None:
        response = client.applications.with_raw_response.process(
            edges=[
                {
                    "from_field": "from_field",
                    "from_node": "from_node",
                    "to_field": "to_field",
                    "to_node": "to_node",
                },
                {
                    "from_field": "from_field",
                    "from_node": "from_node",
                    "to_field": "to_field",
                    "to_node": "to_node",
                },
                {
                    "from_field": "from_field",
                    "from_node": "from_node",
                    "to_field": "to_field",
                    "to_node": "to_node",
                },
            ],
            inputs={"foo": "bar"},
            nodes=[
                {
                    "id": "id",
                    "application_node_schema_id": "text_input_schema",
                },
                {
                    "id": "id",
                    "application_node_schema_id": "text_input_schema",
                },
                {
                    "id": "id",
                    "application_node_schema_id": "text_input_schema",
                },
            ],
            version="V0",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        application = response.parse()
        assert_matches_type(object, application, path=["response"])

    @pytest.mark.skip(reason="additional_properties is an object")
    @parametrize
    def test_streaming_response_process(self, client: SGPClient) -> None:
        with client.applications.with_streaming_response.process(
            edges=[
                {
                    "from_field": "from_field",
                    "from_node": "from_node",
                    "to_field": "to_field",
                    "to_node": "to_node",
                },
                {
                    "from_field": "from_field",
                    "from_node": "from_node",
                    "to_field": "to_field",
                    "to_node": "to_node",
                },
                {
                    "from_field": "from_field",
                    "from_node": "from_node",
                    "to_field": "to_field",
                    "to_node": "to_node",
                },
            ],
            inputs={"foo": "bar"},
            nodes=[
                {
                    "id": "id",
                    "application_node_schema_id": "text_input_schema",
                },
                {
                    "id": "id",
                    "application_node_schema_id": "text_input_schema",
                },
                {
                    "id": "id",
                    "application_node_schema_id": "text_input_schema",
                },
            ],
            version="V0",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            application = response.parse()
            assert_matches_type(object, application, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="prism_binary_unsupported")
    @parametrize
    def test_method_upload_files(self, client: SGPClient) -> None:
        application = client.applications.upload_files(
            files=[b"raw file contents", b"raw file contents", b"raw file contents"],
        )
        assert_matches_type(ApplicationUploadFilesResponse, application, path=["response"])

    @pytest.mark.skip(reason="prism_binary_unsupported")
    @parametrize
    def test_method_upload_files_with_all_params(self, client: SGPClient) -> None:
        application = client.applications.upload_files(
            files=[b"raw file contents", b"raw file contents", b"raw file contents"],
            account_id="account_id",
        )
        assert_matches_type(ApplicationUploadFilesResponse, application, path=["response"])

    @pytest.mark.skip(reason="prism_binary_unsupported")
    @parametrize
    def test_raw_response_upload_files(self, client: SGPClient) -> None:
        response = client.applications.with_raw_response.upload_files(
            files=[b"raw file contents", b"raw file contents", b"raw file contents"],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        application = response.parse()
        assert_matches_type(ApplicationUploadFilesResponse, application, path=["response"])

    @pytest.mark.skip(reason="prism_binary_unsupported")
    @parametrize
    def test_streaming_response_upload_files(self, client: SGPClient) -> None:
        with client.applications.with_streaming_response.upload_files(
            files=[b"raw file contents", b"raw file contents", b"raw file contents"],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            application = response.parse()
            assert_matches_type(ApplicationUploadFilesResponse, application, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_validate(self, client: SGPClient) -> None:
        application = client.applications.validate(
            edges=[
                {
                    "from_field": "from_field",
                    "from_node": "from_node",
                    "to_field": "to_field",
                    "to_node": "to_node",
                },
                {
                    "from_field": "from_field",
                    "from_node": "from_node",
                    "to_field": "to_field",
                    "to_node": "to_node",
                },
                {
                    "from_field": "from_field",
                    "from_node": "from_node",
                    "to_field": "to_field",
                    "to_node": "to_node",
                },
            ],
            nodes=[
                {
                    "id": "id",
                    "application_node_schema_id": "text_input_schema",
                },
                {
                    "id": "id",
                    "application_node_schema_id": "text_input_schema",
                },
                {
                    "id": "id",
                    "application_node_schema_id": "text_input_schema",
                },
            ],
            version="V0",
        )
        assert_matches_type(object, application, path=["response"])

    @parametrize
    def test_method_validate_with_all_params(self, client: SGPClient) -> None:
        application = client.applications.validate(
            edges=[
                {
                    "from_field": "from_field",
                    "from_node": "from_node",
                    "to_field": "to_field",
                    "to_node": "to_node",
                },
                {
                    "from_field": "from_field",
                    "from_node": "from_node",
                    "to_field": "to_field",
                    "to_node": "to_node",
                },
                {
                    "from_field": "from_field",
                    "from_node": "from_node",
                    "to_field": "to_field",
                    "to_node": "to_node",
                },
            ],
            nodes=[
                {
                    "id": "id",
                    "application_node_schema_id": "text_input_schema",
                    "configuration": {"foo": {"value": {}}},
                },
                {
                    "id": "id",
                    "application_node_schema_id": "text_input_schema",
                    "configuration": {"foo": {"value": {}}},
                },
                {
                    "id": "id",
                    "application_node_schema_id": "text_input_schema",
                    "configuration": {"foo": {"value": {}}},
                },
            ],
            version="V0",
            overrides={
                "foo": {
                    "artifact_ids_filter": ["string", "string", "string"],
                    "artifact_name_regex": ["string", "string", "string"],
                    "type": "knowledge_base_schema",
                }
            },
        )
        assert_matches_type(object, application, path=["response"])

    @parametrize
    def test_raw_response_validate(self, client: SGPClient) -> None:
        response = client.applications.with_raw_response.validate(
            edges=[
                {
                    "from_field": "from_field",
                    "from_node": "from_node",
                    "to_field": "to_field",
                    "to_node": "to_node",
                },
                {
                    "from_field": "from_field",
                    "from_node": "from_node",
                    "to_field": "to_field",
                    "to_node": "to_node",
                },
                {
                    "from_field": "from_field",
                    "from_node": "from_node",
                    "to_field": "to_field",
                    "to_node": "to_node",
                },
            ],
            nodes=[
                {
                    "id": "id",
                    "application_node_schema_id": "text_input_schema",
                },
                {
                    "id": "id",
                    "application_node_schema_id": "text_input_schema",
                },
                {
                    "id": "id",
                    "application_node_schema_id": "text_input_schema",
                },
            ],
            version="V0",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        application = response.parse()
        assert_matches_type(object, application, path=["response"])

    @parametrize
    def test_streaming_response_validate(self, client: SGPClient) -> None:
        with client.applications.with_streaming_response.validate(
            edges=[
                {
                    "from_field": "from_field",
                    "from_node": "from_node",
                    "to_field": "to_field",
                    "to_node": "to_node",
                },
                {
                    "from_field": "from_field",
                    "from_node": "from_node",
                    "to_field": "to_field",
                    "to_node": "to_node",
                },
                {
                    "from_field": "from_field",
                    "from_node": "from_node",
                    "to_field": "to_field",
                    "to_node": "to_node",
                },
            ],
            nodes=[
                {
                    "id": "id",
                    "application_node_schema_id": "text_input_schema",
                },
                {
                    "id": "id",
                    "application_node_schema_id": "text_input_schema",
                },
                {
                    "id": "id",
                    "application_node_schema_id": "text_input_schema",
                },
            ],
            version="V0",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            application = response.parse()
            assert_matches_type(object, application, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncApplications:
    parametrize = pytest.mark.parametrize("async_client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="additional_properties is an object")
    @parametrize
    async def test_method_process(self, async_client: AsyncSGPClient) -> None:
        application = await async_client.applications.process(
            edges=[
                {
                    "from_field": "from_field",
                    "from_node": "from_node",
                    "to_field": "to_field",
                    "to_node": "to_node",
                },
                {
                    "from_field": "from_field",
                    "from_node": "from_node",
                    "to_field": "to_field",
                    "to_node": "to_node",
                },
                {
                    "from_field": "from_field",
                    "from_node": "from_node",
                    "to_field": "to_field",
                    "to_node": "to_node",
                },
            ],
            inputs={"foo": "bar"},
            nodes=[
                {
                    "id": "id",
                    "application_node_schema_id": "text_input_schema",
                },
                {
                    "id": "id",
                    "application_node_schema_id": "text_input_schema",
                },
                {
                    "id": "id",
                    "application_node_schema_id": "text_input_schema",
                },
            ],
            version="V0",
        )
        assert_matches_type(object, application, path=["response"])

    @pytest.mark.skip(reason="additional_properties is an object")
    @parametrize
    async def test_method_process_with_all_params(self, async_client: AsyncSGPClient) -> None:
        application = await async_client.applications.process(
            edges=[
                {
                    "from_field": "from_field",
                    "from_node": "from_node",
                    "to_field": "to_field",
                    "to_node": "to_node",
                },
                {
                    "from_field": "from_field",
                    "from_node": "from_node",
                    "to_field": "to_field",
                    "to_node": "to_node",
                },
                {
                    "from_field": "from_field",
                    "from_node": "from_node",
                    "to_field": "to_field",
                    "to_node": "to_node",
                },
            ],
            inputs={"foo": "bar"},
            nodes=[
                {
                    "id": "id",
                    "application_node_schema_id": "text_input_schema",
                    "configuration": {"foo": {"value": {}}},
                },
                {
                    "id": "id",
                    "application_node_schema_id": "text_input_schema",
                    "configuration": {"foo": {"value": {}}},
                },
                {
                    "id": "id",
                    "application_node_schema_id": "text_input_schema",
                    "configuration": {"foo": {"value": {}}},
                },
            ],
            version="V0",
            history=[
                {
                    "request": "request",
                    "response": "response",
                    "session_data": {},
                },
                {
                    "request": "request",
                    "response": "response",
                    "session_data": {},
                },
                {
                    "request": "request",
                    "response": "response",
                    "session_data": {},
                },
            ],
            overrides={
                "foo": {
                    "artifact_ids_filter": ["string", "string", "string"],
                    "artifact_name_regex": ["string", "string", "string"],
                    "type": "knowledge_base_schema",
                }
            },
            stream=True,
        )
        assert_matches_type(object, application, path=["response"])

    @pytest.mark.skip(reason="additional_properties is an object")
    @parametrize
    async def test_raw_response_process(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.applications.with_raw_response.process(
            edges=[
                {
                    "from_field": "from_field",
                    "from_node": "from_node",
                    "to_field": "to_field",
                    "to_node": "to_node",
                },
                {
                    "from_field": "from_field",
                    "from_node": "from_node",
                    "to_field": "to_field",
                    "to_node": "to_node",
                },
                {
                    "from_field": "from_field",
                    "from_node": "from_node",
                    "to_field": "to_field",
                    "to_node": "to_node",
                },
            ],
            inputs={"foo": "bar"},
            nodes=[
                {
                    "id": "id",
                    "application_node_schema_id": "text_input_schema",
                },
                {
                    "id": "id",
                    "application_node_schema_id": "text_input_schema",
                },
                {
                    "id": "id",
                    "application_node_schema_id": "text_input_schema",
                },
            ],
            version="V0",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        application = await response.parse()
        assert_matches_type(object, application, path=["response"])

    @pytest.mark.skip(reason="additional_properties is an object")
    @parametrize
    async def test_streaming_response_process(self, async_client: AsyncSGPClient) -> None:
        async with async_client.applications.with_streaming_response.process(
            edges=[
                {
                    "from_field": "from_field",
                    "from_node": "from_node",
                    "to_field": "to_field",
                    "to_node": "to_node",
                },
                {
                    "from_field": "from_field",
                    "from_node": "from_node",
                    "to_field": "to_field",
                    "to_node": "to_node",
                },
                {
                    "from_field": "from_field",
                    "from_node": "from_node",
                    "to_field": "to_field",
                    "to_node": "to_node",
                },
            ],
            inputs={"foo": "bar"},
            nodes=[
                {
                    "id": "id",
                    "application_node_schema_id": "text_input_schema",
                },
                {
                    "id": "id",
                    "application_node_schema_id": "text_input_schema",
                },
                {
                    "id": "id",
                    "application_node_schema_id": "text_input_schema",
                },
            ],
            version="V0",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            application = await response.parse()
            assert_matches_type(object, application, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="prism_binary_unsupported")
    @parametrize
    async def test_method_upload_files(self, async_client: AsyncSGPClient) -> None:
        application = await async_client.applications.upload_files(
            files=[b"raw file contents", b"raw file contents", b"raw file contents"],
        )
        assert_matches_type(ApplicationUploadFilesResponse, application, path=["response"])

    @pytest.mark.skip(reason="prism_binary_unsupported")
    @parametrize
    async def test_method_upload_files_with_all_params(self, async_client: AsyncSGPClient) -> None:
        application = await async_client.applications.upload_files(
            files=[b"raw file contents", b"raw file contents", b"raw file contents"],
            account_id="account_id",
        )
        assert_matches_type(ApplicationUploadFilesResponse, application, path=["response"])

    @pytest.mark.skip(reason="prism_binary_unsupported")
    @parametrize
    async def test_raw_response_upload_files(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.applications.with_raw_response.upload_files(
            files=[b"raw file contents", b"raw file contents", b"raw file contents"],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        application = await response.parse()
        assert_matches_type(ApplicationUploadFilesResponse, application, path=["response"])

    @pytest.mark.skip(reason="prism_binary_unsupported")
    @parametrize
    async def test_streaming_response_upload_files(self, async_client: AsyncSGPClient) -> None:
        async with async_client.applications.with_streaming_response.upload_files(
            files=[b"raw file contents", b"raw file contents", b"raw file contents"],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            application = await response.parse()
            assert_matches_type(ApplicationUploadFilesResponse, application, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_validate(self, async_client: AsyncSGPClient) -> None:
        application = await async_client.applications.validate(
            edges=[
                {
                    "from_field": "from_field",
                    "from_node": "from_node",
                    "to_field": "to_field",
                    "to_node": "to_node",
                },
                {
                    "from_field": "from_field",
                    "from_node": "from_node",
                    "to_field": "to_field",
                    "to_node": "to_node",
                },
                {
                    "from_field": "from_field",
                    "from_node": "from_node",
                    "to_field": "to_field",
                    "to_node": "to_node",
                },
            ],
            nodes=[
                {
                    "id": "id",
                    "application_node_schema_id": "text_input_schema",
                },
                {
                    "id": "id",
                    "application_node_schema_id": "text_input_schema",
                },
                {
                    "id": "id",
                    "application_node_schema_id": "text_input_schema",
                },
            ],
            version="V0",
        )
        assert_matches_type(object, application, path=["response"])

    @parametrize
    async def test_method_validate_with_all_params(self, async_client: AsyncSGPClient) -> None:
        application = await async_client.applications.validate(
            edges=[
                {
                    "from_field": "from_field",
                    "from_node": "from_node",
                    "to_field": "to_field",
                    "to_node": "to_node",
                },
                {
                    "from_field": "from_field",
                    "from_node": "from_node",
                    "to_field": "to_field",
                    "to_node": "to_node",
                },
                {
                    "from_field": "from_field",
                    "from_node": "from_node",
                    "to_field": "to_field",
                    "to_node": "to_node",
                },
            ],
            nodes=[
                {
                    "id": "id",
                    "application_node_schema_id": "text_input_schema",
                    "configuration": {"foo": {"value": {}}},
                },
                {
                    "id": "id",
                    "application_node_schema_id": "text_input_schema",
                    "configuration": {"foo": {"value": {}}},
                },
                {
                    "id": "id",
                    "application_node_schema_id": "text_input_schema",
                    "configuration": {"foo": {"value": {}}},
                },
            ],
            version="V0",
            overrides={
                "foo": {
                    "artifact_ids_filter": ["string", "string", "string"],
                    "artifact_name_regex": ["string", "string", "string"],
                    "type": "knowledge_base_schema",
                }
            },
        )
        assert_matches_type(object, application, path=["response"])

    @parametrize
    async def test_raw_response_validate(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.applications.with_raw_response.validate(
            edges=[
                {
                    "from_field": "from_field",
                    "from_node": "from_node",
                    "to_field": "to_field",
                    "to_node": "to_node",
                },
                {
                    "from_field": "from_field",
                    "from_node": "from_node",
                    "to_field": "to_field",
                    "to_node": "to_node",
                },
                {
                    "from_field": "from_field",
                    "from_node": "from_node",
                    "to_field": "to_field",
                    "to_node": "to_node",
                },
            ],
            nodes=[
                {
                    "id": "id",
                    "application_node_schema_id": "text_input_schema",
                },
                {
                    "id": "id",
                    "application_node_schema_id": "text_input_schema",
                },
                {
                    "id": "id",
                    "application_node_schema_id": "text_input_schema",
                },
            ],
            version="V0",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        application = await response.parse()
        assert_matches_type(object, application, path=["response"])

    @parametrize
    async def test_streaming_response_validate(self, async_client: AsyncSGPClient) -> None:
        async with async_client.applications.with_streaming_response.validate(
            edges=[
                {
                    "from_field": "from_field",
                    "from_node": "from_node",
                    "to_field": "to_field",
                    "to_node": "to_node",
                },
                {
                    "from_field": "from_field",
                    "from_node": "from_node",
                    "to_field": "to_field",
                    "to_node": "to_node",
                },
                {
                    "from_field": "from_field",
                    "from_node": "from_node",
                    "to_field": "to_field",
                    "to_node": "to_node",
                },
            ],
            nodes=[
                {
                    "id": "id",
                    "application_node_schema_id": "text_input_schema",
                },
                {
                    "id": "id",
                    "application_node_schema_id": "text_input_schema",
                },
                {
                    "id": "id",
                    "application_node_schema_id": "text_input_schema",
                },
            ],
            version="V0",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            application = await response.parse()
            assert_matches_type(object, application, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["AsyncJobListResponse"]


class AsyncJobListResponse(BaseModel):
    id: str
    """The unique identifier of the entity."""

    account_id: str
    """The ID of the account that owns the given entity."""

    created_at: datetime
    """The date and time when the entity was created in ISO format."""

    job_type: Literal[
        "evaluation-auto-eval",
        "evaluation-test-cases-result-gen",
        "evaluation-builder",
        "evaluation-dataset-generation",
        "knowledge_base_upload",
        "application-variant-report-generation",
        "application-interaction-evaluation",
    ]
    """An enumeration."""

    status: str

    updated_at: datetime
    """The date and time when the entity was last updated in ISO format."""

    created_by_user_id: Optional[str] = None
    """The user who originally created the entity."""

    job_metadata: Optional[object] = None

    parent_job_id: Optional[str] = None

    progress: Optional[object] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["SharePointDataSourceConfigParam"]


class SharePointDataSourceConfigParam(TypedDict, total=False):
    client_id: Required[str]
    """Client ID associated with this SharePoint site"""

    site_id: Required[str]
    """
    Site ID for this SharePoint site, can be found at
    https://[hostname].sharepoint.com/sites/[site name]/\\__api/site/id
    """

    source: Required[Literal["SharePoint"]]

    tenant_id: Required[str]
    """Tenant ID that the SharePoint site is within"""

    folder_path: str
    """Nested folder path to read files from the root of the site.

    Please omit the leading slash. Example: 'Documents/sub_directory'
    """

    recursive: bool
    """Recurse through the folder contents, default is True."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.hybrid_evaluation_metrics import HybridEvaluationMetrics

__all__ = ["HybridEvalMetricsResource", "AsyncHybridEvalMetricsResource"]


class HybridEvalMetricsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> HybridEvalMetricsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python#accessing-raw-response-data-eg-headers
        """
        return HybridEvalMetricsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> HybridEvalMetricsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python#with_streaming_response
        """
        return HybridEvalMetricsResourceWithStreamingResponse(self)

    def retrieve(
        self,
        evaluation_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> HybridEvaluationMetrics:
        """
        Get Hybrid Evaluation Metrics

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not evaluation_id:
            raise ValueError(f"Expected a non-empty value for `evaluation_id` but received {evaluation_id!r}")
        return self._get(
            f"/v4/evaluations/{evaluation_id}/hybrid-eval-metrics",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=HybridEvaluationMetrics,
        )


class AsyncHybridEvalMetricsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncHybridEvalMetricsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python#accessing-raw-response-data-eg-headers
        """
        return AsyncHybridEvalMetricsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncHybridEvalMetricsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python#with_streaming_response
        """
        return AsyncHybridEvalMetricsResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        evaluation_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> HybridEvaluationMetrics:
        """
        Get Hybrid Evaluation Metrics

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not evaluation_id:
            raise ValueError(f"Expected a non-empty value for `evaluation_id` but received {evaluation_id!r}")
        return await self._get(
            f"/v4/evaluations/{evaluation_id}/hybrid-eval-metrics",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=HybridEvaluationMetrics,
        )


class HybridEvalMetricsResourceWithRawResponse:
    def __init__(self, hybrid_eval_metrics: HybridEvalMetricsResource) -> None:
        self._hybrid_eval_metrics = hybrid_eval_metrics

        self.retrieve = to_raw_response_wrapper(
            hybrid_eval_metrics.retrieve,
        )


class AsyncHybridEvalMetricsResourceWithRawResponse:
    def __init__(self, hybrid_eval_metrics: AsyncHybridEvalMetricsResource) -> None:
        self._hybrid_eval_metrics = hybrid_eval_metrics

        self.retrieve = async_to_raw_response_wrapper(
            hybrid_eval_metrics.retrieve,
        )


class HybridEvalMetricsResourceWithStreamingResponse:
    def __init__(self, hybrid_eval_metrics: HybridEvalMetricsResource) -> None:
        self._hybrid_eval_metrics = hybrid_eval_metrics

        self.retrieve = to_streamed_response_wrapper(
            hybrid_eval_metrics.retrieve,
        )


class AsyncHybridEvalMetricsResourceWithStreamingResponse:
    def __init__(self, hybrid_eval_metrics: AsyncHybridEvalMetricsResource) -> None:
        self._hybrid_eval_metrics = hybrid_eval_metrics

        self.retrieve = async_to_streamed_response_wrapper(
            hybrid_eval_metrics.retrieve,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from .._models import BaseModel
from .application_edge import ApplicationEdge
from .application_node import ApplicationNode

__all__ = ["ApplicationConfiguration"]


class ApplicationConfiguration(BaseModel):
    edges: List[ApplicationEdge]

    nodes: List[ApplicationNode]

    metadata: Optional[object] = None
    """User defined metadata about the application"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["ConfluenceDataSourceConfig"]


class ConfluenceDataSourceConfig(BaseModel):
    source: Literal["Confluence"]

    space_key: str
    """Confluence space key to retrieve contents from.

    See https://support.atlassian.com/confluence-cloud/docs/choose-a-space-key
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Union, Optional
from typing_extensions import Literal, TypeAlias

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = [
    "ApplicationNodeSchemaRegistryRecord",
    "NodeSchema",
    "NodeSchemaConfiguration",
    "NodeSchemaConfigurationNumberConfigurationFieldSchema",
    "NodeSchemaConfigurationNumberConfigurationFieldSchemaValidationRules",
    "NodeSchemaConfigurationStringConfigurationFieldSchema",
    "NodeSchemaConfigurationStringConfigurationFieldSchemaValidationRules",
    "NodeSchemaConfigurationBoolConfigurationFieldSchema",
    "NodeSchemaConfigurationBoolConfigurationFieldSchemaValidationRules",
    "NodeSchemaInputs",
    "NodeSchemaOutputs",
]


class NodeSchemaConfigurationNumberConfigurationFieldSchemaValidationRules(BaseModel):
    max_value: int

    min_value: int

    required: bool


class NodeSchemaConfigurationNumberConfigurationFieldSchema(BaseModel):
    docs: str

    label: str

    validation_rules: NodeSchemaConfigurationNumberConfigurationFieldSchemaValidationRules

    default: Optional[float] = None

    is_advanced_configuration: Optional[bool] = None

    is_float: Optional[bool] = None

    type: Optional[Literal["TEXT", "STRING_ARRAY", "NUMBER", "BOOL", "CHUNK_ARRAY", "DOCUMENT_ARRAY"]] = None
    """
    Represents the possible types of values for an application configuration, inputs
    and outputs.

    The available types are:

    - TEXT: Represents a single line of text.
    - STRING_ARRAY: Represents an array of strings.
    - NUMBER: Represents a numeric value.
    - BOOL: Represents a boolean value.
    - CHUNK_ARRAY: Represents an array of chunks.
    - DOCUMENT_ARRAY: Represents an array of documents.
    """


class NodeSchemaConfigurationStringConfigurationFieldSchemaValidationRules(BaseModel):
    required: bool

    regex_pattern: Optional[str] = None


class NodeSchemaConfigurationStringConfigurationFieldSchema(BaseModel):
    docs: str

    label: str

    validation_rules: NodeSchemaConfigurationStringConfigurationFieldSchemaValidationRules

    default: Optional[bool] = None

    is_advanced_configuration: Optional[bool] = None

    type: Optional[Literal["TEXT", "STRING_ARRAY", "NUMBER", "BOOL", "CHUNK_ARRAY", "DOCUMENT_ARRAY"]] = None
    """
    Represents the possible types of values for an application configuration, inputs
    and outputs.

    The available types are:

    - TEXT: Represents a single line of text.
    - STRING_ARRAY: Represents an array of strings.
    - NUMBER: Represents a numeric value.
    - BOOL: Represents a boolean value.
    - CHUNK_ARRAY: Represents an array of chunks.
    - DOCUMENT_ARRAY: Represents an array of documents.
    """


class NodeSchemaConfigurationBoolConfigurationFieldSchemaValidationRules(BaseModel):
    required: bool


class NodeSchemaConfigurationBoolConfigurationFieldSchema(BaseModel):
    docs: str

    label: str

    validation_rules: NodeSchemaConfigurationBoolConfigurationFieldSchemaValidationRules

    default: Optional[bool] = None

    is_advanced_configuration: Optional[bool] = None

    type: Optional[Literal["TEXT", "STRING_ARRAY", "NUMBER", "BOOL", "CHUNK_ARRAY", "DOCUMENT_ARRAY"]] = None
    """
    Represents the possible types of values for an application configuration, inputs
    and outputs.

    The available types are:

    - TEXT: Represents a single line of text.
    - STRING_ARRAY: Represents an array of strings.
    - NUMBER: Represents a numeric value.
    - BOOL: Represents a boolean value.
    - CHUNK_ARRAY: Represents an array of chunks.
    - DOCUMENT_ARRAY: Represents an array of documents.
    """


NodeSchemaConfiguration: TypeAlias = Union[
    NodeSchemaConfigurationNumberConfigurationFieldSchema,
    NodeSchemaConfigurationStringConfigurationFieldSchema,
    NodeSchemaConfigurationBoolConfigurationFieldSchema,
]


class NodeSchemaInputs(BaseModel):
    docs: str

    label: str

    name: str

    required: bool

    type: Literal["TEXT", "STRING_ARRAY", "NUMBER", "BOOL", "CHUNK_ARRAY", "DOCUMENT_ARRAY"]
    """
    Represents the possible types of values for an application configuration, inputs
    and outputs.

    The available types are:

    - TEXT: Represents a single line of text.
    - STRING_ARRAY: Represents an array of strings.
    - NUMBER: Represents a numeric value.
    - BOOL: Represents a boolean value.
    - CHUNK_ARRAY: Represents an array of chunks.
    - DOCUMENT_ARRAY: Represents an array of documents.
    """


class NodeSchemaOutputs(BaseModel):
    docs: str

    label: str

    name: str

    type: Literal["TEXT", "STRING_ARRAY", "NUMBER", "BOOL", "CHUNK_ARRAY", "DOCUMENT_ARRAY"]
    """
    Represents the possible types of values for an application configuration, inputs
    and outputs.

    The available types are:

    - TEXT: Represents a single line of text.
    - STRING_ARRAY: Represents an array of strings.
    - NUMBER: Represents a numeric value.
    - BOOL: Represents a boolean value.
    - CHUNK_ARRAY: Represents an array of chunks.
    - DOCUMENT_ARRAY: Represents an array of documents.
    """


class NodeSchema(BaseModel):
    configuration: Dict[str, NodeSchemaConfiguration]

    inputs: Dict[str, NodeSchemaInputs]

    outputs: Dict[str, NodeSchemaOutputs]


class ApplicationNodeSchemaRegistryRecord(BaseModel):
    id: str

    name: str

    node_schema: NodeSchema = FieldInfo(alias="nodeSchema")

    tags: List[Literal["INPUT_NODE", "OUTPUT_NODE", "RAG_NODE"]]

    type: Literal[
        "TEXT_INPUT_NODE",
        "TEXT_OUTPUT_NODE",
        "KNOWLEDGE_BASE_NODE",
        "RERANKER_NODE",
        "PROMPT_ENGINEERING_NODE",
        "COMPLETION_MODEL_NODE",
        "EXTERNAL_ENDPOINT_NODE",
        "DOCUMENT_INPUT_NODE",
        "MAP_REDUCE_NODE",
        "DOCUMENT_SEARCH_NODE",
        "DOCUMENT_PROMPT_NODE",
    ]
    """An enumeration."""

    version: Literal["OFFLINE", "V0", "AGENTS_SERVICE"]
    """
    An enum representing the version states of an application and its nodes'
    schemas. Attributes: V0: The initial version of an application schema.
    AGENTS_SERVICE: Application schema references egp_services definition.
    """
